from invenio_records_resources.services.records.schema import BaseRecordSchema as InvenioBaseRecordSchema
from marshmallow import ValidationError
from marshmallow import validate as ma_validate
import marshmallow as ma
from marshmallow import fields as ma_fields
from marshmallow_utils import fields as mu_fields
from marshmallow_utils import schemas as mu_schemas

{% for imp in imports %}
{% if imp.alias %}
from {{imp.import_path|package_name}} import {{imp.import_path|base_name}} as {{imp.alias}}
{% else %}
from {{imp.import_path|package_name}} import {{imp.import_path|base_name}}
{% endif %}
{% endfor %}

{% for cls in generated_classes %}
{{cls}}
{% endfor %}